@@ -26,8 +26,9 @@ test_that("Tests that the predictInfo function works", {
 
   skip_if_not_mac()
   expect_equal(all.equal(info$obsInfo[[1]]$Weight[1:10],
-                         c(0.068042104, 0.015729129, 0.008917891, 0.008753721,
-                           0.053599199, 0.008646022, 0.030503379, 0.022268962,
-                           0.012211569, 0.005780545)),TRUE)
+                         c(0.065719889164469164066773, 0.016520037897421121358965, 0.008480538754331324105551,
+                           0.007944978989019234674740, 0.052003063531344574654813, 0.008815330945284504879367,
+                           0.033636028293209586925716, 0.024207273402751239982367,
+                           0.011876750142739506133083, 0.005217297104913203205367), tolerance = 1e-8),TRUE)
 
 })
